"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.NotionService = void 0;
const fetchData_1 = require("../../util/fetchData");
class NotionService {
    reporter;
    parallelLimit;
    enableCaching;
    cache;
    limiter;
    constructor(options) {
        this.reporter = options.reporter;
        this.parallelLimit = options.parallelLimit || 5; // 기본 동시 요청 수
        this.enableCaching = options.enableCaching !== false; // 기본값은 캐싱 활성화
        this.cache = new Map();
        this.initLimiter();
    }
    async initLimiter() {
        try {
            const pLimitModule = await Promise.resolve().then(() => __importStar(require("p-limit")));
            const pLimit = pLimitModule.default;
            this.limiter = pLimit(this.parallelLimit);
        }
        catch (error) {
            this.reporter.error(`Failed to initialize p-limit: ${error}`);
            this.limiter = this.createSimpleLimiter(this.parallelLimit);
        }
    }
    createSimpleLimiter(limit) {
        let running = 0;
        const queue = [];
        const next = () => {
            running--;
            if (queue.length > 0) {
                const run = queue.shift();
                if (run)
                    run();
            }
        };
        return (fn) => {
            return new Promise((resolve, reject) => {
                const run = () => {
                    running++;
                    fn().then(resolve).catch(reject).finally(next);
                };
                if (running < limit) {
                    run();
                }
                else {
                    queue.push(run);
                }
            });
        };
    }
    /**
     * 데이터베이스 쿼리
     */
    async queryDatabase(databaseId, body = {}) {
        const cacheKey = `database-${databaseId}-${JSON.stringify(body)}`;
        if (this.enableCaching && this.cache.has(cacheKey)) {
            this.reporter.info(`[CACHE] Using cached database query for ${databaseId}`);
            return this.cache.get(cacheKey);
        }
        const databaseUrl = `databases/${databaseId}/query`;
        try {
            const result = await (0, fetchData_1.fetchPostWithRetry)(databaseUrl, body);
            if (this.enableCaching) {
                this.cache.set(cacheKey, result);
            }
            this.reporter.info(`[SUCCESS] Database query ${databaseId} - results: ${result?.results?.length || 0}`);
            return result;
        }
        catch (error) {
            this.reporter.error(`[ERROR] Failed to query database ${databaseId}: ${error}`);
            throw error;
        }
    }
    /**
     * 페이지의 자식 블록 가져오기
     */
    async getPageBlocks(pageId) {
        const cacheKey = `page-blocks-${pageId}`;
        if (this.enableCaching && this.cache.has(cacheKey)) {
            this.reporter.info(`[CACHE] Using cached page blocks for ${pageId}`);
            return this.cache.get(cacheKey);
        }
        const pageUrl = `blocks/${pageId}/children?page_size=100`;
        try {
            const data = await (0, fetchData_1.fetchGetWithRetry)(pageUrl);
            const blocks = data.results;
            if (this.enableCaching) {
                this.cache.set(cacheKey, blocks);
            }
            return blocks;
        }
        catch (error) {
            this.reporter.error(`[ERROR] Failed to get page blocks ${pageId}: ${error}`);
            throw error;
        }
    }
    /**
     * 여러 페이지의 블록 병렬 처리
     */
    async getMultiplePagesBlocks(pageIds) {
        if (!this.limiter) {
            await this.initLimiter();
        }
        this.reporter.info(`[NOTION] Fetching blocks for ${pageIds.length} pages in parallel (limit: ${this.parallelLimit})`);
        const tasks = pageIds.map((pageId) => this.limiter(async () => {
            try {
                const blocks = await this.getPageBlocks(pageId);
                return { pageId, blocks };
            }
            catch (error) {
                this.reporter.warn(`[WARNING] Failed to fetch blocks for page ${pageId}: ${error}`);
                return { pageId, blocks: [] };
            }
        }));
        const results = await Promise.all(tasks);
        return results.reduce((acc, { pageId, blocks }) => {
            acc[pageId] = blocks;
            return acc;
        }, {});
    }
    /**
     * 캐시 초기화
     */
    clearCache() {
        this.reporter.info(`[NOTION] Clearing service cache - entries: ${this.cache.size}`);
        this.cache.clear();
    }
    /**
     * 병렬 처리 제한 설정
     */
    async setParallelLimit(limit) {
        this.parallelLimit = limit;
        await this.initLimiter();
        this.reporter.info(`[NOTION] Updated parallel request limit to ${limit}`);
    }
}
exports.NotionService = NotionService;
